@@ -26,6 +26,13 @@ try:
     __version__ = importlib_metadata.version("loggerhead")
 except importlib_metadata.PackageNotFoundError:
     # Support running tests from the build tree without installation.
-    __version__ = None
+    import os
+
+    try:
+        from setuptools.config.setupcfg import read_configuration
+    except ModuleNotFoundError:
+        from setuptools.config import read_configuration
+    cfg = read_configuration(os.path.join(os.path.dirname(__file__), "..", "setup.cfg"))
+    __version__ = cfg["metadata"]["version"]
 __revision__ = None
 required_breezy = (3, 1)
