@@ -13,7 +13,7 @@
 #
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+# Foundation, Inc., 51 Franklin Street, Suite 500, Boston, MA 02110-1335  USA
 #
 
 """
@@ -26,20 +26,17 @@ once a revision is committed in bazaar, it never changes, so once we have
 cached a change, it's good forever.
 """
 
-import cPickle
 import marshal
 import os
+import pickle
 import tempfile
 import zlib
-
-try:
-    from sqlite3 import dbapi2
-except ImportError:
-    from pysqlite2 import dbapi2
+from sqlite3 import dbapi2
 
 # We take an optimistic approach to concurrency here: we might do work twice
 # in the case of races, but not crash or corrupt data.
 
+
 def safe_init_db(filename, init_sql):
     # To avoid races around creating the database, we create the db in
     # a temporary file and rename it into the ultimate location.
@@ -52,35 +49,33 @@ def safe_init_db(filename, init_sql):
     con.close()
     os.rename(temp_path, filename)
 
-class FakeShelf(object):
 
+class FakeShelf(object):
     def __init__(self, filename):
         create_table = not os.path.exists(filename)
         if create_table:
             safe_init_db(
-                filename, "create table RevisionData "
-                "(revid binary primary key, data binary)")
+                filename,
+                "create table RevisionData (revid binary primary key, data binary)",
+            )
         self.connection = dbapi2.connect(filename)
         self.cursor = self.connection.cursor()
 
     def _create_table(self, filename):
         con = dbapi2.connect(filename)
         cur = con.cursor()
-        cur.execute(
-            "create table RevisionData "
-            "(revid binary primary key, data binary)")
+        cur.execute("create table RevisionData (revid binary primary key, data binary)")
         con.commit()
         con.close()
 
     def _serialize(self, obj):
-        return dbapi2.Binary(cPickle.dumps(obj, protocol=2))
+        return dbapi2.Binary(pickle.dumps(obj, protocol=2))
 
     def _unserialize(self, data):
-        return cPickle.loads(str(data))
+        return pickle.loads(str(data))
 
     def get(self, revid):
-        self.cursor.execute(
-            "select data from revisiondata where revid = ?", (revid, ))
+        self.cursor.execute("select data from revisiondata where revid = ?", (revid,))
         filechange = self.cursor.fetchone()
         if filechange is None:
             return None
@@ -91,7 +86,8 @@ class FakeShelf(object):
         try:
             self.cursor.execute(
                 "insert into revisiondata (revid, data) values (?, ?)",
-                (revid, self._serialize(object)))
+                (revid, self._serialize(object)),
+            )
             self.connection.commit()
         except dbapi2.IntegrityError:
             # If another thread or process attempted to set the same key, we
@@ -99,58 +95,53 @@ class FakeShelf(object):
             pass
 
 
-class FileChangeCache(object):
-
-    def __init__(self, cache_path):
-
-        if not os.path.exists(cache_path):
-            os.mkdir(cache_path)
-
-        self._changes_filename = os.path.join(cache_path, 'filechanges.sql')
-
-    def get_file_changes(self, entry):
-        cache = FakeShelf(self._changes_filename)
-        changes = cache.get(entry.revid)
-        if changes is None:
-            changes = self.history.get_file_changes_uncached(entry)
-            cache.add(entry.revid, changes)
-        return changes
-
-
 class RevInfoDiskCache(object):
     """Like `RevInfoMemoryCache` but backed in a sqlite DB."""
 
     def __init__(self, cache_path):
         if not os.path.exists(cache_path):
             os.mkdir(cache_path)
-        filename = os.path.join(cache_path, 'revinfo.sql')
+        filename = os.path.join(cache_path, "revinfo.sql")
         create_table = not os.path.exists(filename)
         if create_table:
             safe_init_db(
-                filename, "create table Data "
-                "(key binary primary key, revid binary, data binary)")
+                filename,
+                "create table Data (key binary primary key, revid binary, data binary)",
+            )
         self.connection = dbapi2.connect(filename)
         self.cursor = self.connection.cursor()
 
     def get(self, key, revid):
+        if not isinstance(key, bytes):
+            raise TypeError(key)
+        if not isinstance(revid, bytes):
+            raise TypeError(revid)
         self.cursor.execute(
-            "select revid, data from data where key = ?", (dbapi2.Binary(key),))
+            "select revid, data from data where key = ?", (dbapi2.Binary(key),)
+        )
         row = self.cursor.fetchone()
         if row is None:
             return None
         elif str(row[0]) != revid:
             return None
         else:
-            return marshal.loads(zlib.decompress(row[1]))
+            try:
+                return marshal.loads(zlib.decompress(row[1]))
+            except (EOFError, ValueError, TypeError):
+                return None
 
     def set(self, key, revid, data):
+        if not isinstance(key, bytes):
+            raise TypeError(key)
+        if not isinstance(revid, bytes):
+            raise TypeError(revid)
         try:
-            self.cursor.execute(
-                'delete from data where key = ?', (dbapi2.Binary(key), ))
-            blob = zlib.compress(marshal.dumps(data))
+            self.cursor.execute("delete from data where key = ?", (dbapi2.Binary(key),))
+            blob = zlib.compress(marshal.dumps(data, 2))
             self.cursor.execute(
                 "insert into data (key, revid, data) values (?, ?, ?)",
-                map(dbapi2.Binary, [key, revid, blob]))
+                list(map(dbapi2.Binary, [key, revid, blob])),
+            )
             self.connection.commit()
         except dbapi2.IntegrityError:
             # If another thread or process attempted to set the same key, we
