@@ -14,27 +14,25 @@
 #
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+# Foundation, Inc., 51 Franklin Street, Suite 500, Boston, MA 02110-1335  USA
 #
 
-from loggerhead.controllers import TemplatedBranchView
+from ..controllers import TemplatedBranchView
 
 
-class AtomUI (TemplatedBranchView):
-
-    template_path = 'loggerhead.templates.atom'
+class AtomUI(TemplatedBranchView):
+    template_name = "atom"
 
     def get_values(self, path, kwargs, headers):
         history = self._history
-        revid = self.get_revid()
-        pagesize = int(20)#self._branch.config.get('pagesize', '20'))
+        pagesize = int(20)  # self._branch.config.get('pagesize', '20'))
 
         revid_list = history.get_file_view(history.last_revid, None)
         entries = list(history.get_changes(list(revid_list)[:pagesize]))
 
-        headers['Content-Type'] = 'application/atom+xml'
+        headers["Content-Type"] = "application/atom+xml"
         return {
-            'changes': entries,
-            'updated': entries[0].date.isoformat() + 'Z',
-            'history': self._history,
+            "changes": entries,
+            "updated": entries[0].utc_date.isoformat(),
+            "history": self._history,
         }
