@@ -1,11 +1,12 @@
-'''Profiling middleware for paste.'''
-import cgi
+"""Profiling middleware for Paste."""
+
 import threading
 
-from bzrlib.lsprof import profile
+from breezy.lsprof import profile
+
 
 class LSProfMiddleware(object):
-    '''Paste middleware for profiling with lsprof.'''
+    """Paste middleware for profiling with lsprof."""
 
     def __init__(self, app, global_conf=None):
         self.app = app
@@ -17,7 +18,7 @@ class LSProfMiddleware(object):
         try:
             return list(app_iter)
         finally:
-            if getattr(app_iter, 'close', None):
+            if getattr(app_iter, "close", None):
                 app_iter.close()
 
     def __call__(self, environ, start_response):
@@ -26,8 +27,7 @@ class LSProfMiddleware(object):
         try:
             ret, stats = profile(self.__run_app, environ, start_response)
             self.__count += 1
-            stats.save("%d-stats.callgrind" % self.__count, format="callgrind")
+            stats.save("%d-stats.callgrind" % (self.__count,), format="callgrind")
             return ret
         finally:
             self.lock.release()
-
