@@ -1,4 +1,4 @@
-# Copyright 2006, 2010 Canonical Ltd
+# Copyright 2006, 2010, 2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -15,13 +15,27 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
 
-def load_tests(standard_tests, module, loader):
-    standard_tests.addTests(loader.loadTestsFromModuleNames([
-        (__name__ + '.' + x) for x in [
-            'test_controllers',
-            'test_corners',
-            'test_simple',
-            'test_templating',
-            'test_util',
-        ]]))
-    return standard_tests
+from __future__ import absolute_import
+
+
+def test_suite():
+    import unittest
+
+    loader = unittest.TestLoader()
+    return loader.loadTestsFromNames(
+        [
+            (__name__ + "." + x)
+            for x in [
+                "test_controllers",
+                "test_corners",
+                "test_history",
+                "test_http_head",
+                "test_load_test",
+                "test_simple",
+                "test_revision_ui",
+                "test_templating",
+                "test_util",
+                "test_highlight",
+            ]
+        ]
+    )
